// gog_samples.cpp : Defines the entry point for the application.
//

#include "stdafx.h"
#include "clf_application.h"
#include "clf_input.h"
#include "clf_timer.h"
#include "clf_util.h"
#include "clf_math.h"

// windows stuff
static LRESULT CALLBACK	WndProc(HWND, UINT, WPARAM, LPARAM);

static clf_application* g_app = NULL;

// program entry point
static LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	return g_app->WndProc(hWnd, message, wParam, lParam);
}

int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPTSTR lpCmdLine, int nCmdShow)
{
	UNREFERENCED_PARAMETER(hPrevInstance);
	UNREFERENCED_PARAMETER(lpCmdLine);

	init_math_lib();
	g_app->hInstance = hInstance;

	return g_app->run();
}

static bool change_screen_resolution (int width, int height, int bitsPerPixel)
{
	DEVMODE dmScreenSettings;
	ZeroMemory (&dmScreenSettings, sizeof (DEVMODE));
	dmScreenSettings.dmSize				= sizeof (DEVMODE);
	dmScreenSettings.dmPelsWidth		= width;						// Select Screen Width
	dmScreenSettings.dmPelsHeight		= height;						// Select Screen Height
	dmScreenSettings.dmBitsPerPel		= bitsPerPixel;			// Select Bits Per Pixel
	dmScreenSettings.dmFields			= DM_BITSPERPEL | DM_PELSWIDTH | DM_PELSHEIGHT;
	if (ChangeDisplaySettings (&dmScreenSettings, CDS_FULLSCREEN) != DISP_CHANGE_SUCCESSFUL)
	{
		return false;													// Display Change Failed, Return False
	}
	return true;														// Display Change Was Successful, Return True
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int clf_application::run()
{
	MSG msg;

	// Perform application initialization:
	TCHAR szWindowClass[100];			// the main window class name
	strcpy(szWindowClass, "GusOnGames");

	hDC=0;							// Device Context
	hRC=0;							// Rendering Context
	isMinimized = FALSE;			// is window visible or minimized
	can_take_fullscreen_msg = true;

	WNDCLASSEX wcex;
	ZeroMemory (&wcex, sizeof (WNDCLASSEX));

	wcex.cbSize = sizeof(WNDCLASSEX);

	wcex.style			= CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
	wcex.lpfnWndProc	= ::WndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInstance;
	wcex.hIcon			= LoadIcon(hInstance, MAKEINTRESOURCE(IDI_GOG_SAMPLES));
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH)(COLOR_APPWORKSPACE);
	wcex.lpszMenuName	= MAKEINTRESOURCE(IDC_GOG_SAMPLES);
	wcex.lpszClassName	= szWindowClass;
	wcex.hIconSm		= LoadIcon(wcex.hInstance, MAKEINTRESOURCE(IDI_SMALL));

	if (RegisterClassEx (&wcex) == 0)
	{
		MessageBox (HWND_DESKTOP, "RegisterClassEx Failed!", "Error", MB_OK | MB_ICONEXCLAMATION);
		return 0;
	}

	// zero the keystatus buffers
	memset(keyDown, 0, sizeof(keyDown));

	init_application();

	/// {{
	bool bAppRunning = true;
	while (bAppRunning)
	{
		if (win.init.isFullScreen)
		{
			win.hWnd = create_window (win.init.width, win.init.height, win.init.bitsPerPixel, win.init.title, hInstance, szWindowClass, 1);
		}
		else
		{
			win.hWnd = create_window (win.init.w_width, win.init.w_height, win.init.bitsPerPixel, win.init.title, hInstance, szWindowClass, 1);
		}

		if (!win.hWnd)
			return 0;

		InitDirectInput( win.hWnd, true, true, win.init.directMouse, win.init.directJoystick );

		glewInit();

		// loop regardles of vsync
		wglSwapIntervalEXT(0);

		graphics_init();    // init upon startup

		// start the timer before we enter the main loop
		get_global_timer()->start();

		can_take_fullscreen_msg = true;  // accept toggle full screen command if it comes

		// Main message loop:
		bool bLooping = true;
		while (bLooping)
		{
			// check for windows message and precess them
			if (PeekMessage (&msg, NULL, 0, 0, PM_REMOVE) != 0)
			{
				if (msg.message == WM_QUIT)
				{
					bLooping = false;
					bAppRunning = false;
				}
				if (msg.message == WM_FULLSCREEN)
				{
					win.init.isFullScreen = !win.init.isFullScreen;
					bLooping = false;
				}

				TranslateMessage(&msg);
				DispatchMessage(&msg);
			}
			else  // no messages, just loop for next frame
			{
				if (isMinimized)		// if window is minimized
				{
					WaitMessage ();		// yeld back to system, do not waste processing power
				}
				else
				{
					UpdateInput(win.hWnd);
					// update the scene based on the elapsed time since last update
					frame_move((clFloat)get_global_timer()->get_elapsed_time());
					frame_render();		// render scene
					SwapBuffers (hDC);	// Swap Buffers (Double Buffering)
				}
			}
		}

		graphics_terminate();								// User Defined Cleanup

		FreeDirectInput();

		destroy_window();
	}
	/// }}

	exit_application();

	// stop the clock
	get_global_timer()->stop();

	return (int) msg.wParam;
}

LRESULT clf_application::WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	PAINTSTRUCT ps;
	HDC hdc;
	static int mouse_set = 0;
	static int last_mouse_x;
	static int last_mouse_y;

	switch (message)
	{
	case WM_SIZE:	// window size change
		switch (wParam)	// evaluate
		{
		case SIZE_MINIMIZED:	// minimized?
			isMinimized = true;	// set isMinimized to true
			return 0;

		case SIZE_MAXIMIZED:	// maximized?
			isMinimized = false;// set isMinimized to false
			window_resize (LOWORD (lParam), HIWORD (lParam));	// recalc view - LoWord=Width, HiWord=Height
			return 0;

		case SIZE_RESTORED:		// restored?
			isMinimized = false;// set isMinimized to false
			window_resize (LOWORD (lParam), HIWORD (lParam));	// recalc view - LoWord=Width, HiWord=Height
			return 0;
		}
		break;
	case WM_PAINT:
		hdc = BeginPaint(hWnd, &ps);
		EndPaint(hWnd, &ps);
		break;

	case WM_KEYDOWN:												// Update Keyboard Buffers For clf_keys Pressed
		if ((wParam >= 0) && (wParam <= 255))						// Is Key (wParam) In A Valid Range?
		{
			keyDown [wParam] = true;					// Set The Selected Key (wParam) To True
			return 0;												// Return
		}
		break;															// Break

	case WM_KEYUP:													// Update Keyboard Buffers For clf_keys Released
		if ((wParam >= 0) && (wParam <= 255))						// Is Key (wParam) In A Valid Range?
		{
			keyDown [wParam] = false;					// Set The Selected Key (wParam) To False
			return 0;												// Return
		}
		break;															// Break

	case WM_DESTROY:
		if (can_take_fullscreen_msg)  // check if this is generated by a change resolution message
			PostQuitMessage(0);
		break;
	default:
		return DefWindowProc(hWnd, message, wParam, lParam);
	}
	return 0;
}

// This function Creates Our OpenGL Window
HWND clf_application::create_window (int wid, int hei, int bitsPerPixel, const char* title, HINSTANCE hInstance, const char* classname, int stencilBuffer)
{
	DWORD windowStyle = WS_OVERLAPPEDWINDOW;				// define our window style
	DWORD windowExtendedStyle = WS_EX_APPWINDOW;			// define the window's extended style

	int width = wid;
	int height = hei;

	RECT windowRect = {0, 0, width, height};				// define our window coordinates


	if (win.init.isFullScreen)
	{
		change_screen_resolution(width, height, bitsPerPixel);
		ShowCursor (false);											// Turn Off The Cursor
		windowStyle = WS_POPUP;										// Set The WindowStyle To WS_POPUP (Popup Window)
		windowExtendedStyle |= WS_EX_TOPMOST;						// Set The Extended Window Style To WS_EX_TOPMOST
	}
	else
	{
		// adjust window, account for window borders
		AdjustWindowRectEx (&windowRect, windowStyle, 0, windowExtendedStyle);
	}

	// create the opengl window
	win.hWnd = CreateWindowEx (windowExtendedStyle,	// extended style
		classname,								// class name
		title,									// window title
		windowStyle,							// window style
		0, 0,									// window x,y position
		windowRect.right - windowRect.left,		// window width
		windowRect.bottom - windowRect.top,		// window height
		HWND_DESKTOP,							// desktop is window's parent
		0,										// no menu
		hInstance,								// pass the window instance
		0);

	if (win.hWnd == 0)								// was window creation a success?
	{
		return win.hWnd;							// if not return false
	}

	hDC = GetDC(win.hWnd);							// grab a device context for this window
	if (hDC == 0)								// did we get a device context?
	{
		// Failed
		DestroyWindow (win.hWnd);					// destroy the window
		win.hWnd = 0;								// zero the window handle
		return win.hWnd;							// return false
	}

	PIXELFORMATDESCRIPTOR pfd =								// pfd tells windows how we want things to be
	{
		sizeof (PIXELFORMATDESCRIPTOR),						// size of this pixel format descriptor
		1,													// version number
		PFD_DRAW_TO_WINDOW |								// format must support window
		PFD_SUPPORT_OPENGL |								// format must support opengl
		PFD_DOUBLEBUFFER,									// must support double buffering
		PFD_TYPE_RGBA,										// request an rgba format
		bitsPerPixel,										// select our color depth
		0, 0, 0, 0, 0, 0,									// color bits ignored
		0,													// no alpha buffer
		0,													// shift bit ignored
		0,													// no accumulation buffer
		0, 0, 0, 0,											// accumulation bits ignored
		16,													// 16bit z-buffer (depth buffer)  
		stencilBuffer,										// stencil buffer
		0,													// no auxiliary buffer
		PFD_MAIN_PLANE,										// main drawing layer
		0,													// reserved
		0, 0, 0												// layer masks ignored
	};

	GLuint PixelFormat = ChoosePixelFormat (hDC, &pfd);	// find a compatible pixel format
	if (PixelFormat == 0)								// did we find a compatible format?
	{
		// Failed
		ReleaseDC (win.hWnd, hDC);							// release our device context
		hDC = 0;										// zero the device context
		DestroyWindow (win.hWnd);							// destroy the window
		win.hWnd = 0;										// zero the window handle
		return win.hWnd;									// return false
	}

	if (SetPixelFormat (hDC, PixelFormat, &pfd) == false)// try to set the pixel format
	{
		// Failed
		ReleaseDC (win.hWnd, hDC);							// release our device context
		hDC = 0;										// zero the device context
		DestroyWindow (win.hWnd);							// destroy the window
		win.hWnd = 0;										// zero the window handle
		return win.hWnd;									// return false
	}

	hRC = wglCreateContext (hDC);						// try to get a rendering context
	if (hRC == 0)										// did we get a rendering context?
	{
		// Failed
		ReleaseDC (win.hWnd, hDC);							// release our device context
		hDC = 0;										// zero the device context
		DestroyWindow (win.hWnd);							// destroy the window
		win.hWnd = 0;										// zero the window handle
		return win.hWnd;									// return false
	}

	// make the rendering context our current rendering context
	if (wglMakeCurrent (hDC, hRC) == false)				// failed
	{
		wglDeleteContext (hRC);							// delete the rendering context
		hRC = 0;										// zero the rendering context
		ReleaseDC (win.hWnd, hDC);							// release our device context
		hDC = 0;										// zero the device context
		DestroyWindow (win.hWnd);							// destroy the window
		win.hWnd = 0;										// zero the window handle
		return win.hWnd;									// return false
	}

	ShowWindow (win.hWnd, SW_NORMAL);						// make the window visible
	isMinimized = false;								// set isMinimized to false

	window_resize (width, height);						// reshape our gl window
	UpdateWindow(win.hWnd);

	return win.hWnd;										// window creating was a success
}

bool clf_application::destroy_window ()								// destroy the opengl window & release resources
{
	if (win.hWnd != 0)										// does the window have a handle?
	{	
		if (hDC != 0)									// does the window have a device context?
		{
			wglMakeCurrent (hDC, 0);					// set the current active rendering context to zero
			if (hRC != 0)								// does the window have a rendering context?
			{
				wglDeleteContext (hRC);					// release the rendering context
				hRC = 0;								// zero the rendering context

			}
			ReleaseDC (win.hWnd, hDC);						// release the device context
			hDC = 0;									// zero the device context
		}
		DestroyWindow (win.hWnd);							// destroy the window
		win.hWnd = 0;										// zero the window handle
	}

	if (win.init.isFullScreen)										// Is Window In Fullscreen Mode
	{
		ChangeDisplaySettings (NULL,0);									// Switch Back To Desktop Resolution
		ShowCursor (true);												// Show The Cursor
	}

	return true;										// return true
}

clf_application::clf_application()
{
	if (g_app == NULL)
		g_app = this;
	else
		abort();

	ZeroMemory (&win, sizeof (clf_window));			// Make Sure Memory Is Zeroed
	win.init.title			= "clf_ge core sample";					// Window Title
	win.init.width			= 1680;									// Full Screen Width
	win.init.height			= 1050;									// Full Screen Height
	win.init.bitsPerPixel	= 32;									// Bits Per Pixel
	win.init.w_width    = 840;									// Window Width
	win.init.w_height   = 525;									// Window Height
	win.init.isFullScreen	= false;							// Fullscreen? (Set To TRUE)
}

clf_application::~clf_application()
{
}

bool clf_application::init_application()
{
	return true;
}

void clf_application::exit_application()
{
}

bool clf_application::graphics_init()
{
	return true;
}

bool clf_application::graphics_reset()
{
	return true;
}

void clf_application::graphics_terminate()
{
}

void clf_application::frame_move(clFloat fElapsed)
{
	if (isKeyPressed(DIK_ESCAPE)) // Is ESC Being Pressed?
		terminate_application ();						// Terminate The Program
	if (isKeyPressed(DIK_F2)) // Is F2 Being Pressed?
		toggle_fullscreen ();						// Toggle full screen
}

void clf_application::frame_render()
{
}

void clf_application::window_resize(int w, int h)
{
}

void clf_application::terminate_application ()							// Terminate The Application
{
	PostMessage (win.hWnd, WM_QUIT, 0, 0);							// Send A WM_QUIT Message
}

void clf_application::toggle_fullscreen ()								// Toggle Fullscreen/Windowed
{
	if (can_take_fullscreen_msg)
	{
		PostMessage (win.hWnd, WM_FULLSCREEN, 0, 0);				// Send A WM_FULLSCREEN Message
		can_take_fullscreen_msg = false; // reject toggle full screen command until this is executed

	}
}
