#pragma once

#include "../resource.h"
#include "clf_dispatch.h"

#define WM_FULLSCREEN (WM_USER+1)									// Application Define Message For Toggling

struct clf_window_pref{						// Creation Preferences
	char*				title;							// Title
	int					width;							// Full Screen Width
	int					height;							// Full Screen Height
	int					bitsPerPixel;				// Full Screen Bits Per Pixel
	int					w_width;						// Windowed Width
	int					w_height;						// Windowed Height
	bool				isFullScreen;				// FullScreen?
	int         stencilBuffer;      // stencilBuffer?
	// Direct Input needs
	bool directMouse;
	bool directJoystick;
} ;									// clf_window_pref

struct clf_window{									// Contains Information Vital To A Window
	HWND				hWnd;						// Window Handle
	HDC					hDC;						// Device Context
	HGLRC				hRC;						// Rendering Context
	bool				isVisible;					// Window Visible?
	clf_window_pref		init;						// Window Init
} ;										// clf_window


class clf_application : public clf_dispatch {
private:
	const char*		className;						// Application ClassName

	// static HINSTANCE hInst;						// current instance
	HDC   hDC;							// Device Context
	HGLRC hRC;							// Rendering Context
	bool isMinimized;			// is window visible or minimized

	bool can_take_fullscreen_msg;
	bool keyDown [256];

	void toggle_fullscreen ();			// Toggle Fullscreen / Windowed Mode
	HWND create_window (int wid, int hei, int bitsPerPixel, const char* title, HINSTANCE hInstance, const char* classname, int stencilBuffer);
	bool destroy_window ();

protected:
	clf_window   win;

public:
	HINSTANCE		hInstance;						// Application Instance

	clf_application();
	virtual ~clf_application();

	virtual bool init_application();          // Application Init (before OpenGL init)
	virtual void exit_application();          // Application Term
	virtual bool graphics_init();     // Graphics Init (after OpenGL init, safe to do OpenGL stuff)
	virtual void graphics_terminate();		 // just before widow is destroyed (called even in reslution change)

	// called after graphics_init when prgram starts or after window creation in reslution change
	// used to reload geometry and textures into OpenGL
	virtual bool graphics_reset();

	virtual void frame_move(clFloat fElapsed);
	virtual void frame_render();
	virtual void window_resize(int w, int h);     // called when window is resized to update display settings

	int run();
	void terminate_application ();		// Terminate The Application
	LRESULT WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
};
