#ifndef __CLF_CAMERA__
#define __CLF_CAMERA__

#include "clf_math.h"

class cCamera{
public:
	clf_vector3D vLocation;
	clf_vector3D vForward;
	clf_vector3D vUp;

	cCamera(){
		// setup a default viewer
		setup(clf_vector3D(0,0,3),        // position viewer
					clf_vector3D(0,0,0),        // setup the point of interest (where we are looking at)
					clf_vector3D(0,1,0));       // which way is up
	}
	cCamera(cCamera& cp){
		vLocation = cp.vLocation;
		vUp = cp.vUp;
		vForward = cp.vForward;
	}
	virtual ~cCamera(){}
	virtual void move_to(clf_vector3D& pos){
		vLocation = pos;
	}
	virtual void look_at(clf_vector3D& lookat){
		vForward = lookat-vLocation;
		vForward.normalize();
	}
	virtual void orient(clf_vector3D& up){
		vUp = up;
	}

	void apply();
	void apply_mirror();
	void moveby(clFloat fwdx,clFloat fwdy,clFloat fwdz);
	void rotate(clFloat x, clFloat y, clFloat z);
	void rotate_vec(clf_vector3D& v, clFloat x, clFloat y, clFloat z);
	void rotate_locX(clFloat angle);
	void rotate_locY(clFloat angle);
	void rotate_locZ(clFloat angle);
	void setup(clf_vector3D& pos, clf_vector3D& lookat, clf_vector3D& up);

	clf_matrix4x4 rotation_matrix();
};

class clf_stage{
public:
	clf_stage(){
		left = bottom = 0;
		near_plane = .2f;
		far_plane = 1000.f;
		perspective_angle = 30;
	}
	~clf_stage(){
	}

	void resize(int l, int b, int w, int h){
		left = l;
		bottom = b;
		width = w;
		height = h;
	}

	void set_perspective(clFloat angle, clFloat np, clFloat fp){
		perspective_angle = angle;
		near_plane = np;
		far_plane = fp;
	}

	void viewport(){
		glViewport (left, bottom, (GLsizei)(width), (GLsizei)(height));				// reset the current viewport
	}
	void perspective(){
		clFloat w = (clFloat)width;
		clFloat h = (clFloat)height;
		gluPerspective (perspective_angle, w/h, near_plane, far_plane);	// calculate the aspect ratio of the window
	}

	void prepare(){
		glViewport (left, bottom, (GLsizei)(width), (GLsizei)(height));				// reset the current viewport
		glMatrixMode (GL_PROJECTION);										// select the projection matrix
		glLoadIdentity ();													// reset the projection matrix

		clFloat w = (clFloat)width;
		clFloat h = (clFloat)height;

		// the two paths are exactly the same (internally)
		// in the first we calculate the dimensions of the near plane visible area
		// which requires some elementary but sometimes forgotten trigonometry
		// the second is provided by the utility functions in OpenGL
#if 1
		clFloat r = w / h;
		clFloat persp_angle = perspective_angle/2.f;      // half of the perspective angle goes to one side
		clFloat ys = near_plane*(clFloat)tan(persp_angle*pi/180.f);
		clFloat xs = ys*r;
		glFrustum( -xs, xs, -ys, ys, near_plane, far_plane );
#else
		gluPerspective (perspective_angle, w/h, near_plane, far_plane);	// calculate the aspect ratio of the window
#endif
	}

	int left, bottom;
	int width, height;
	clFloat near_plane, far_plane;
	clFloat perspective_angle;
};

#endif // __CLF_CAMERA__
