#include "stdafx.h"
#include "clf_font.h"

#ifdef _WINDOWS
//PFNGLWINDOWPOS2IPROC glWindowPos2i = NULL;
#endif

clf_fontMapper clf_font_manager::m_fontMap;
clf_font* clf_font_manager::selected = NULL;

clf_font_manager::clf_font_manager()
{
}

clf_font_manager::~clf_font_manager()
{
}

void clf_font_manager::add_font(const std::string name, clf_font* fnt)
{
	clf_fontMapper::iterator it = clf_font_manager::m_fontMap.find(name);
	if (it == m_fontMap.end())
		m_fontMap.insert(std::make_pair(name, fnt));
}

clf_font* clf_font_manager::get_font(const std::string  name)
{
	clf_fontMapper::iterator it = clf_font_manager::m_fontMap.find(name);
	if (it == m_fontMap.end())
		return NULL;
	return it->second;
}

clf_font* clf_font_manager::select_font(const std::string id)
{
	selected = get_font(id);
	return selected;
}

clf_font* clf_font_manager::create_font_2d(char *fontName, char *sysFontName, int fontSize)
{
	clf_font* gfnt = new clf_font(sysFontName, fontSize);
	add_font(fontName, gfnt);
	return NULL;
}

clf_font* clf_font_manager::create_font_3d(char *fontName, char *sysFontName, int fontSize, clFloat depth)
{
	clf_font* menu_font = new clf_font(sysFontName, fontSize, depth);
	add_font(fontName, menu_font);
	return NULL;
}

void clf_font_manager::clf_reset()
{
	clf_fontMapper::iterator it;
	for (it=m_fontMap.begin(); it!=m_fontMap.end(); ++it)
	{
		clf_font* pf = it->second;
		pf->clf_reset();
	}
}

void clf_font_manager::clear()
{
	clf_fontMapper::iterator it;
	for (it=m_fontMap.begin(); it!=m_fontMap.end(); ++it)
	{
		clf_font* pf = it->second;
		delete pf;
	}
	m_fontMap.clear();
}

static clf_font_manager fManager;

clf_font_manager& get_font_manager()
{
	return fManager;
}

////////////////////////////////////////////////////////////////
clf_font::clf_font() : m_fontName(""), m_fontSize(0), m_depth(0), m_type(fontUnknown)
{
	clf_reset();
}

clf_font::clf_font(char *fontName, int fontSize) : m_fontName(fontName), m_fontSize(fontSize), m_depth(0), m_type(font2d)
{
	clf_reset();
}

clf_font::clf_font(char *fontName, int fontSize, clFloat depth) : m_fontName(fontName), m_fontSize(fontSize), m_depth(depth), m_type(font3d)
{
	clf_reset();
}

clf_font::~clf_font()
{
	clear();
}

void clf_font::clf_reset()
{
	init();
	if (m_type == font2d)
		build_2d(m_fontName.c_str(), m_fontSize);
	else if (m_type == font3d)
		build_3d(m_fontName.c_str(), m_fontSize, m_depth);
}

void clf_font::clear()
{
	glDeleteLists(listBase, 256);
}

void clf_font::init()
{
	m_useScreenPosition = true;
	screenX = 0;
	screenY = 0;
	xpos = 0.0;
	ypos = 0.0;
	zpos = 0.0;
}

void clf_font::build_3d(const char *name, int size, clFloat depth)
{
	HFONT hFont;
	HDC hDC = wglGetCurrentDC();
	listBase = glGenLists(256);      // create storage for 256 characters

	// ask windows to create the outline of the font
	if (strcmp(name, "symbol") == 0)
	{
		hFont = CreateFont(-size, 0,0,0,FW_BOLD, FALSE, FALSE, FALSE, SYMBOL_CHARSET,
			OUT_TT_PRECIS, CLIP_DEFAULT_PRECIS, ANTIALIASED_QUALITY,
			FF_DONTCARE | DEFAULT_PITCH, name);
	}
	else
	{
		hFont = CreateFont(size, 0, 0, 0, FW_NORMAL, FALSE, FALSE, FALSE, ANSI_CHARSET, 
			OUT_TT_PRECIS, CLIP_DEFAULT_PRECIS, ANTIALIASED_QUALITY,
			FF_DONTCARE | DEFAULT_PITCH, name);
	}

	// on faillure return
	if (!hFont)
		return;

	// and now ask the system to generate the geometry and store it in the draw lists
	HFONT oFont = (HFONT)SelectObject(hDC, hFont);
	wglUseFontOutlines(hDC, 0, 255, listBase, 0.0f, depth, WGL_FONT_POLYGONS, gmf);
	SelectObject(hDC, oFont);
	DeleteObject(hFont);
}

void clf_font::build_2d(const char *name, int size)
{
	HFONT hFont;
	HDC hDC = wglGetCurrentDC();
	listBase = glGenLists(256);

	if (strcmp(name, "symbol") == 0)
	{
		hFont = CreateFont(-size, 0,0,0,FW_BOLD, FALSE, FALSE, FALSE, SYMBOL_CHARSET,
			OUT_TT_PRECIS, CLIP_DEFAULT_PRECIS, ANTIALIASED_QUALITY,
			FF_DONTCARE | DEFAULT_PITCH, name);
	}
	else
	{
		hFont = CreateFont(-size, 0,0,0,FW_NORMAL, FALSE, FALSE, FALSE, ANSI_CHARSET,
			OUT_TT_PRECIS, CLIP_DEFAULT_PRECIS, ANTIALIASED_QUALITY, 
			FF_DONTCARE | DEFAULT_PITCH, name);
	}

	HFONT oFont = (HFONT)SelectObject(hDC, hFont);
	wglUseFontBitmaps(hDC, 0, 256, listBase);
	SelectObject(hDC, oFont);
	DeleteObject(hFont);
}

void clf_font::print_3d(int alignment, const char *str, ...)
{
	clFloat length = 0;
	char text[512];
	va_list args;

	if ((str == NULL))
		return;

	va_start(args, str);
	vsprintf(text, str, args);
	va_end(args);

	glColor4f(r,g,b,a);

	// center the text
	for (unsigned int loop=0;loop<(strlen(text));loop++)	// find length of text
	{
		length+=gmf[text[loop]].gmfCellIncX;		        // increase length by character's width
	}
	switch (alignment)
	{
	case ALIGN_CENTER:
		glTranslatef(-length/2,0.0f,0.0f);                  // translate to center text
		break;
	case ALIGN_LEFT:
		break;
	case ALIGN_RIGHT:
		glTranslatef(-length,0.0f,0.0f);                  // translate to center text
		break;
	}

	// draw the text
	glListBase(listBase);
	glCallLists((GLsizei)strlen(text), GL_UNSIGNED_BYTE, text);
}

void clf_font::print_2d(const char *str, ...)
{
	char text[512];
	va_list args;

	if (str == NULL)
		return;

	va_start(args, str);
	vsprintf(text, str, args);
	va_end(args);

	//glColor4f(r,g,b,a);

#ifdef _WINDOWS
	if (m_useScreenPosition)
		glWindowPos2i(screenX, screenY);
#endif

	glListBase(listBase);
	glCallLists((GLsizei)strlen(text), GL_UNSIGNED_BYTE, text);
}
