/* fonts for text output */
#ifndef __FONT_H__
#define __FONT_H__

#include <map>

#define ALIGN_CENTER 0
#define ALIGN_LEFT   1
#define ALIGN_RIGHT  2

// simple font class supporting 2d and 3d fonts
class clf_font{
	unsigned int listBase;				// display list base
	GLYPHMETRICSFLOAT gmf[256];			// holds orientation and placement

	unsigned int texID;		// font texture id
	unsigned int callList;	// font display list
	clFloat r, g, b, a;		// RGBA
	int screenX, screenY;	// screen coordinates
	clFloat xpos, ypos, zpos;	// 3d coordinates
	std::string m_fontName;
	int m_fontSize;
	clFloat m_depth;
	enum e_fontType {fontUnknown, font2d, font3d};
	e_fontType m_type;
	bool m_useScreenPosition;

	void init();

public:
	// 3d font constructor (depth)
	clf_font(char *fontName, int fontSize, clFloat depth);
	// 2d font constructor (no depth)
	clf_font(char *fontName, int fontSize);
	// default constructor
	clf_font();
	~clf_font();

	// print 3d (aligned left/center/right), syntax like printf
	void print_3d(int alignment, const char *str, ...);
	// print 2d (no align), syntax like printf
	void print_2d(const char *str, ...);

	// actual creation functions
	void build_3d(const char *name, int size, clFloat depth);
	void build_2d(const char *name, int size);

	// explicit cleanup
	void clear();

	// text positioning
	// 3d if space coordinates
	void set_position_3d(clFloat x, clFloat y, clFloat z) { xpos = x; ypos = y; zpos = z; }
	// 2d positioning is in pixels
	void set_position_2d(int x, int y) { screenX = x; screenY = y; }
	void use_screen_position(bool buse){
		m_useScreenPosition = buse;
	}

	// text color
	void set_color_RGB(clFloat red, clFloat green, clFloat blue) { r = red; g = green; b = blue; a = 1.0; }
	void set_color_RGBA(clFloat red, clFloat green, clFloat blue, clFloat alpha) { r = red; g = green; b = blue; a = alpha; }

	void clf_reset();
};

typedef std::map<std::string, clf_font*> clf_fontMapper;

// font manager
class clf_font_manager
{
	static clf_fontMapper m_fontMap;
	static clf_font* selected;
public:
	clf_font_manager();
	~clf_font_manager();
	void add_font(const std::string id, clf_font* fnt);
	clf_font* get_font(const std::string id);
	clf_font* select_font(const std::string id);

	clf_font* create_font_2d(char *fontName, char *sysFontName, int fontSize);
	clf_font* create_font_3d(char *fontName, char *sysFontName, int fontSize, clFloat depth);

	void clf_reset();

	clf_font* select_font(){
		return selected;
	}
	void clear();
};

clf_font_manager& get_font_manager();

#endif // __FONT_H__
