#include "stdafx.h"
#include "clf_camera.h"
#include "clf_light.h"
#include "clf_shader.h"
#include "clf_skybox.h"
#include "clf_effects.h"
#include "clf_world.h"
#include "clf_mesh.h"
#include "clf_font.h"
#include "clf_util.h"


#include "clf_application.h"
#include "clf_menu.h"
#include "clf_input.h"

extern clf_font* gfnt;

clf_menu::clf_menu() : clf_dispatch(), option(0)
{
	menu_font = NULL;
	m_key_state[0] = m_key_state[1] = m_key_state[2] = false; // up, down, enter
	xpos = -6;
	ypos = 5;
	zpos = 6;
	xstep = 0.f;
	ystep = 1.25;
	zstep = 0.f;
}

clf_menu::~clf_menu()
{
}

void clf_menu::add_option(char* text, int id)
{
	menu_entry me;
	me.entry_id = id;
	me.entry_text = std::string(text);
	entries.push_back(me);
}

void clf_menu::set_pos(clFloat xp, clFloat yp, clFloat zp)
{
	xpos = xp;
	ypos = yp;
	zpos = zp;
}

void clf_menu::set_step(clFloat xs, clFloat ys, clFloat zs)
{
	xstep = xs;
	ystep = ys;
	zstep = zs;
}

void clf_menu::set_menu_callBack(clf_functor* pf)
{
	menu_handler = pf;
}

void clf_menu::frame_render()
{
	// clear the screen
	// drawing the skybox means we do not have to waste time clearing the screen
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	// clear screen and stencil buffer

	glMatrixMode (GL_PROJECTION);									// select the projection matrix
	glLoadIdentity ();														// reset the projection matrix

	// viewport and perspective
	m_stage.prepare();

	glMatrixMode (GL_MODELVIEW);								// select the modelview matrix
	glLoadIdentity ();													// reset

	glPushMatrix();

	// position the viewer
	m_cam.apply();

	glPushAttrib(GL_ALL_ATTRIB_BITS);

	glEnable(GL_NORMALIZE);
	// position the light
	m_light.enable();

	// reapply the light
	m_light.apply();

	int i;
	clFloat y = ypos;
	for (i=0; i<(int)entries.size(); i++)
	{
		glPushMatrix();
		if (i==option)
		{
			glTranslatef(xpos, y, zpos-2);
			menu_font->set_color_RGB(.9f,.1f,.1f);
		}
		else
		{
			glTranslatef(xpos, y, zpos);
			menu_font->set_color_RGB(.1f,.5f,.9f);
		}
		menu_font->print_3d(ALIGN_CENTER, entries[i].entry_text.c_str());
		glPopMatrix();
		y -= ystep;
	}

	glPopMatrix();
	glPopAttrib();
}

bool clf_menu::graphics_init()
{
	clf_dispatch::graphics_init();
	m_cam.setup(clf_vector3D(0,0,-15),       // position viewer
							clf_vector3D(0,0,0),        // setup the point of interest (where we are looking at)
							clf_vector3D(0,1,0));       // which way is up
	menu_font = get_font_manager().get_font("Bookman20");
	m_light.set_ambient(0.25f, 0.25f, 0.25f, 1.f);
	m_light.set_diffuse(0.7f, 0.7f, 0.7f, 1.f);
	m_light.set_specular(1.f, 1.f, 1.f, 1.f);
	m_light.set_position(0.f, 0.f, -100.f, 0.f);
	return menu_font != NULL;
}

void clf_menu::window_resize (int width, int height)
{
	m_stage.resize(0,0,width, height);
}

void clf_menu::frame_move(clFloat fElapsed)
{
	clf_dispatch::frame_move(fElapsed);
	if (isKeyPressed(DIK_UP) && !m_key_state[0])
	{
		option--;
		if (option < 0)
			option = (int)entries.size()-1;
	}
	else if (isKeyPressed(DIK_DOWN) && !m_key_state[1])
	{
		option++;
		if (option >= (int)entries.size())
			option = 0;
	}
	else if (isKeyPressed(DIK_RETURN) && !m_key_state[2])
	{
		if (menu_handler)
			menu_handler->invoke(entries[option].entry_id);
	}

	m_key_state[0] = isKeyPressed(DIK_UP)?1:0;
	m_key_state[1] = isKeyPressed(DIK_DOWN)?1:0;
	m_key_state[2] = isKeyPressed(DIK_RETURN)?1:0;
}
